@@ -17,7 +17,6 @@ use termion::raw::IntoRawMode;
 use termion::{clear, cursor, screen};
 
 pub fn run(arg: PathBuf) -> Result<(), MyError> {
-    env_logger::init();
     debug!("Initial setup starts.");
 
     let mut config_dir_path =
@@ -31,7 +30,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
     make_session(&session_file_path).unwrap_or_else(|_| panic!("Cannot make session file."));
 
     if !&arg.exists() {
-        println!("Invalid path: {}", &arg.display());
+        println!("Invalid path: {}\n`fx -h` shows help.", &arg.display());
         return Ok(());
     }
 
@@ -47,7 +46,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
 
     print!("{}", cursor::Hide);
 
-    state.update_list();
+    state.update_list()?;
     clear_and_show(&state.current_dir);
     state.list_up(nums.skip);
 
@@ -204,7 +203,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                             print_warning(e, y);
                                             continue;
                                         }
-                                        state.update_list();
+                                        state.update_list()?;
                                         clear_and_show(&state.current_dir);
                                         nums.reset();
                                         state.list_up(nums.skip);
@@ -255,7 +254,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                             print_warning(e, y);
                                             continue;
                                         }
-                                        state.update_list();
+                                        state.update_list()?;
 
                                         match c_memo_v.pop() {
                                             Some(memo) => {
@@ -321,7 +320,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         print_warning(e, y);
                                         continue;
                                     }
-                                    state.update_list();
+                                    state.update_list()?;
                                     nums = memo.num;
                                     clear_and_show(&state.current_dir);
                                     state.list_up(nums.skip);
@@ -333,7 +332,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         print_warning(e, y);
                                         continue;
                                     }
-                                    state.update_list();
+                                    state.update_list()?;
                                     match pre.file_name() {
                                         Some(name) => {
                                             let mut new_pos = 0;
@@ -580,7 +579,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         count += 1;
                                     }
                                     clear_and_show(&state.current_dir);
-                                    state.update_list();
+                                    state.update_list()?;
                                     state.list_up(nums.skip);
 
                                     let duration = duration_to_string(start.elapsed());
@@ -654,7 +653,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                             state.sort_by = SortKey::Name;
                         }
                     }
-                    state.update_list();
+                    state.update_list()?;
                     clear_and_show(&state.current_dir);
                     state.list_up(0);
                     nums.reset();
@@ -706,7 +705,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         }
 
                                         clear_and_show(&state.current_dir);
-                                        state.update_list();
+                                        state.update_list()?;
                                         state.list_up(nums.skip);
                                         let cursor_pos = if state.list.is_empty() {
                                             STARTING_POINT
@@ -786,7 +785,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                     }
 
                     clear_and_show(&state.current_dir);
-                    state.update_list();
+                    state.update_list()?;
                     state.list_up(nums.skip);
 
                     let duration = duration_to_string(start.elapsed());
@@ -838,7 +837,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                     }
 
                                     clear_and_show(&state.current_dir);
-                                    state.update_list();
+                                    state.update_list()?;
                                     state.list_up(nums.skip);
 
                                     print!("{}", cursor::Hide);
@@ -1054,12 +1053,26 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
 
                                     if command == vec!['q'] {
                                         break 'main;
-                                    } else if command == vec!['e'] {
-                                        state.update_list();
-                                        clear_and_show(&state.current_dir);
-                                        state.list_up(0);
-                                        print!("{}", cursor::Hide);
+                                    } else if command == vec!['c', 'd'] || command == vec!['z'] {
+                                        p_memo_v = Vec::new();
+                                        c_memo_v = Vec::new();
+                                        state.current_dir = dirs::home_dir().unwrap();
                                         nums.reset();
+                                        if let Err(e) = state.update_list() {
+                                            print_warning(e, y);
+                                            break 'command;
+                                        }
+                                        clear_and_show(&state.current_dir);
+                                        state.list_up(nums.skip);
+                                        print!("{}", cursor::Hide);
+                                        state.move_cursor(&nums, STARTING_POINT);
+                                        break 'command;
+                                    } else if command == vec!['e'] {
+                                        nums.reset();
+                                        state.update_list()?;
+                                        clear_and_show(&state.current_dir);
+                                        state.list_up(nums.skip);
+                                        print!("{}", cursor::Hide);
                                         state.move_cursor(&nums, STARTING_POINT);
                                         break 'command;
                                     } else if command == vec!['h'] {
@@ -1094,6 +1107,52 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         }
                                     }
 
+                                    if (c == "cd" || c == "z") && args.is_empty() {
+                                        p_memo_v = Vec::new();
+                                        c_memo_v = Vec::new();
+                                        state.current_dir = dirs::home_dir().unwrap();
+                                        nums.reset();
+                                        state.update_list()?;
+                                        clear_and_show(&state.current_dir);
+                                        state.list_up(nums.skip);
+                                        print!("{}", cursor::Hide);
+                                        state.move_cursor(&nums, STARTING_POINT);
+                                        break 'command;
+                                    }
+
+                                    if c == "z" && args.len() == 1 {
+                                        let output = std::process::Command::new("zoxide")
+                                            .args(["query", args[0].trim()])
+                                            .output()?;
+                                        let output = output.stdout;
+                                        if !output.is_empty() {
+                                            let target_dir = std::str::from_utf8(&output);
+                                            match target_dir {
+                                                Err(e) => {
+                                                    print!("{}", cursor::Hide);
+                                                    print_warning(e, y);
+                                                    break 'command;
+                                                }
+                                                Ok(target_dir) => {
+                                                    let target_path =
+                                                        PathBuf::from(target_dir.trim());
+                                                    print_warning(target_path.to_str().unwrap(), y);
+                                                    state.current_dir =
+                                                        target_path.canonicalize()?;
+                                                    nums.reset();
+                                                    state.update_list()?;
+                                                    p_memo_v = Vec::new();
+                                                    c_memo_v = Vec::new();
+                                                    clear_and_show(&state.current_dir);
+                                                    state.list_up(nums.skip);
+                                                    print!("{}", cursor::Hide);
+                                                    state.move_cursor(&nums, STARTING_POINT);
+                                                    break 'command;
+                                                }
+                                            }
+                                        }
+                                    }
+
                                     if c == "empty" && args.is_empty() {
                                         print_warning(WHEN_EMPTY, y);
                                         screen.flush()?;
@@ -1137,7 +1196,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         );
                                         if state.current_dir == state.trash_dir {
                                             clear_and_show(&state.current_dir);
-                                            state.update_list();
+                                            state.update_list()?;
                                             state.list_up(nums.skip);
                                             state.move_cursor(&nums, STARTING_POINT);
                                         } else {
@@ -1156,7 +1215,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         print!("{}", screen::ToAlternateScreen);
 
                                         clear_and_show(&state.current_dir);
-                                        state.update_list();
+                                        state.update_list()?;
                                         state.list_up(nums.skip);
 
                                         print!("{}", cursor::Hide,);
@@ -1166,7 +1225,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                     print!("{}", screen::ToAlternateScreen);
 
                                     clear_and_show(&state.current_dir);
-                                    state.update_list();
+                                    state.update_list()?;
                                     state.list_up(nums.skip);
 
                                     print!("{}", cursor::Hide);
@@ -1271,7 +1330,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                 // Show/hide hidden files or directories
                 Key::Backspace => {
                     state.show_hidden = !state.show_hidden;
-                    state.update_list();
+                    state.update_list()?;
                     clear_and_show(&state.current_dir);
                     state.list_up(0);
                     nums.reset();
@@ -1289,3 +1348,20 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
     state.write_session(session_file_path)?;
     Ok(())
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn zoxide_test() {
+        let output = std::process::Command::new("zoxide")
+            .args(["query", "dotfiles"])
+            .output()
+            .unwrap();
+        let stdout = std::str::from_utf8(&output.stdout).unwrap().trim();
+        println!("{stdout}");
+        let path = PathBuf::from(stdout);
+        println!("{:?}", path.canonicalize());
+    }
+}
